"""Sample code for MLTDR."""

import sys
import ctypes
import math
import csv
import time     # time.sleep
import timeit
import matplotlib.pyplot as plt

from pymltdrapi import pymltdr

__version__ = "6.1.2"


##############################################################################
"""  Edit parameters for your instance """
##############################################################################

# MLTDR IP address (ex: 192.168.0.1)
IPADDRESS = "172.16.222.26"                                         

PULSEMODE = pymltdr.MLTDRPULSE_MODE.TDR_MODE                        # Pulse mode    (For TDR mode, predefined settings are applied i.e. parameter values with '*' are ignored)
CHANNELSCONFIG = (                                                  # Channels configuration array
     1,                 # Ch Tx1 ID
     1,                 # Ch Tx2 ID
     1,                 # Ch Tx3 ID
     1,                 # Ch Tx4 ID
     0,                 # Ch Rx1 from Ch Tx2 (negative value corresponds to -<Ch Tx2 ID>)
     0,                 # Ch Rx2 from Ch Tx1 (negative value corresponds to -<Ch Tx1 ID>)
     0,                 # Ch Rx3 from Ch Tx3 (negative value corresponds to -<Ch Tx3 ID>)
     0)                 # Ch Rx4 from Ch Tx4 (negative value corresponds to -<Ch Tx4 ID>)
SAMPLESCOUNT = 200000                                              # Samples count*
LINERATE = 26.5625                                                 # Line rate*
NUMAVERAGES = 1                                                    # Number of averages
SIGNALCODING = pymltdr.MLTDRSIGNALCODING.SC_NRZ                    # SignalCoding enum*
TXPATTERN =(                                                       # TX-Pattern enum*
    pymltdr.MLTDRPattern.PRBS13,
    pymltdr.MLTDRPattern.PRBS13,
    pymltdr.MLTDRPattern.PRBS13,
    pymltdr.MLTDRPattern.PRBS13)
SAMPLINGFREQ = 0.0                                                 # Sampling frequency
MAINTAP = (                                                        # Main tap array*
  800,                  # Ch Tx1 Main Tap
    0,                  # Ch Tx2 Main Tap
    0,                  # Ch Tx3 Main Tap
    0)                  # Ch Tx4 Main Tap

PREEMPHASIS = (                                                    # Pre-emphasis array*
    0,                  # Ch Tx1 Pre-emphasis
    0,                  # Ch Tx2 Pre-emphasis
    0,                  # Ch Tx3 Pre-emphasis
    0)                  # Ch Tx4 Pre-emphasis

POSTEMPHASIS = (                                                   # Post-emphasis array*
    0,                  # Ch Tx1 Post-emphasis
    0,                  # Ch Tx2 Post-emphasis
    0,                  # Ch Tx3 Post-emphasis
    0)                  # Ch Tx4 Post-emphasis

#  If pre,post, and main values are 0 for a specific channel, API will use default configuration (MainTap=799, PreEmphasis=-153, PostEmphasis=-48)
 
OUTPUTLEVEL = (                                                    # Output Level array*
    100,                # Ch Tx1 Output Level 
    100,                # Ch Tx2 Output Level 
    100,                # Ch Tx3 Output Level 
    100)                # Ch Tx4 Output Level 

OUTEREYELEVEL = (                                                  # Outer Eye Level array*
    2000,               # Ch Tx1 Outer Eye Level
    2000,               # Ch Tx2 Outer Eye Level
    2000,               # Ch Tx3 Outer Eye Level
    2000)               # Ch Tx4 Outer Eye Level

INNEREYELEVEL = (                                                  # Inner Eye Level array*
    1000,               # Ch Tx1 Inner Eye Level
    1000,               # Ch Tx2 Inner Eye Level
    1000,               # Ch Tx3 Inner Eye Level
    1000)               # Ch Tx4 Inner Eye Level

USERDEFINEDPATTERNLOW = (                                          # UserDefinedPatternLow array*
    0x00ff00ff00ff00ff, # Ch Tx1 UserDefinedPatternLow(64 bits)
    0x00ff00ff00ff00ff, # Ch Tx2 UserDefinedPatternLow(64 bits)
    0,                  # Ch Tx3 UserDefinedPatternLow(64 bits)
    0)                  # Ch Tx4 UserDefinedPatternLow(64 bits)

USERDEFINEDPATTERNMID = (                                          # UserDefinedPatternMid array used in Pam4 mode only*
    0,                  # Ch Tx1 UserDefinedPatternMid(64 bits)
    0,                  # Ch Tx2 UserDefinedPatternMid(64 bits)
    0,                  # Ch Tx3 UserDefinedPatternMid(64 bits)
    0)                  # Ch Tx4 UserDefinedPatternMid(64 bits) 
  
REPETITIONLOW = (                                                  # UserDefinedPatternLow repetition*
    1,                  # Ch Tx1 UserDefinedPatternMid repetition
    1,                  # Ch Tx2 UserDefinedPatternMid repetition
    1,                  # Ch Tx3 UserDefinedPatternMid repetition
    1)                  # Ch Tx4 UserDefinedPatternMid repetition

REPETITIONMID = (                                                  # UserDefinedPatternMid repetition*
    0,                  # Ch Tx1 UserDefinedPatternMid repetition
    0,                  # Ch Tx2 UserDefinedPatternMid repetition
    0,                  # Ch Tx3 UserDefinedPatternMid repetition
    0)                  # Ch Tx4 UserDefinedPatternMid repetition
          
IMPEDANCEOFFSET =(                                                 # User defined Impedance Offset
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0)

SMOOTHINGWINDOWSIZE = 1                                            # Sparam smoothing window size
TXCHANNELS = (0, 1, 2, 3)                                          # Bert desired Tx channels index (e.g. Channel 1 -> 0)
AVGTYPE = pymltdr.MLTDRAVERAGINGTYPE.Hardware                      # Averaging type
PRESETINDEX = 10                                                   # Default -1 if no preset available. Presets are only added for 4035 revC and revD (e.g. Index 10 for TDR mode). Index is ignored if preset is not supported
VELOCITYFACTOR = 0.8                                               # The ratio of actual speed to the speed of light -> between 0 and 1

SAVECSV = False                                                    # Save CSV scope pattern data
TDRDATAOUTPUT = "tdr.csv"                                          # CSV scope pattern data output path

ENABLETIMELOGGING = False                                          # Enables function execution time logging 
SCOPENOISE = 20                                                    # Scope total noise. It can be measured using 'MeasureScopeVrms' without any input signal.

TDRMASK_MINX = 30                                                  # Minimum X percentage for TDR Mask (0->100)
TDRMASK_MAXX = 70                                                  # Maximum X percentage for TDR Mask (0->100)
TDRMASK_MINY = 90                                                  # Minimum Y value for TDR Mask
TDRMASK_MAXY = 110                                                 # Maximum Y value for TDR Mask
#############################################################################

def timeIt(myCall, *args):
    """Time a call.

    Args:
        myCall (:obj:`callable`): Callable object to measure execution time.
        argsList: Optional arbitrary list of call arguments.
        returnIt (bool): Optional flag to enable returning execution return value.

    Returns:
        :obj:`tuple`:
            1-tuple containing the elapsed time if `returnIt` is not set
            or 2-tuple containing the execution return value and the elapsed time otherwise.

    Raises:
        :obj:`TypeError`: Invalid call.

    """
    if not callable(myCall):
        raise TypeError
    start = time.time()
    retVal = myCall(*args)
    end = time.time()
    elapsed = end - start;
    #  Returns
    return (retVal, elapsed) 


def main():
    """Main function."""
    MLTDR = pymltdr.MLTDRAPI()   # Creates instance
    is_running_ber = False

    try:
        #  Connects to device
        if ENABLETIMELOGGING:
            success, timing = timeIt(lambda: MLTDR.Connect(IPADDRESS))
            print("Connect execution time: %f seconds" %timing)
        else:
            success = MLTDR.Connect(IPADDRESS)
        
        if not success:
            raise Exception("Failed to connect to %s!" % IPADDRESS)
        print("Connected")
        
        #  Configures
        presets = [PRESETINDEX]
        MLTDR.InitializeInstance("./", (ctypes.c_int * 1)(*presets), 1)
        ChannelsConfig = (ctypes.c_int * pymltdr.MAXCHANNELS)(*CHANNELSCONFIG)
        conf = pymltdr.mltdrConfigurationParams(PULSEMODE, ChannelsConfig, SAMPLESCOUNT, LINERATE, NUMAVERAGES, TXPATTERN, SIGNALCODING, SAMPLINGFREQ, MAINTAP, PREEMPHASIS, POSTEMPHASIS,
                OUTPUTLEVEL, OUTEREYELEVEL, INNEREYELEVEL, USERDEFINEDPATTERNLOW, USERDEFINEDPATTERNMID, REPETITIONLOW, REPETITIONMID, False, False, SCOPENOISE, AVGTYPE, PRESETINDEX)
        result = (ctypes.c_double * 4)()  # Array of size 3 (time base, frequency resolution, Sampling Frequency)
        
        if ENABLETIMELOGGING:
            success, timing = timeIt(lambda: MLTDR.Configure(conf, result))
            print("Configure execution time: %f seconds" %timing)
        else:
            success = MLTDR.Configure(conf, result)

        if not success:
            raise Exception("Configuration failed!")
        FrequencyResolution = int(result[1])
        Timebase = result[0]
        TimeResolution = int(result[2]) # Configured number of samples. (Used mainly in TDR mode)
        print("Configured. Timebase: %.2f, Frequency Resolution: %s, Sampling Frequency %s" % (Timebase, FrequencyResolution, result[3]))
        #############################################################################
        channels = (ctypes.c_int * pymltdr.NUMTXCHANNELS)(*TXCHANNELS)       

        #############################################################################
        if PULSEMODE == pymltdr.MLTDRPULSE_MODE.TDR_MODE:
            #   'TDR_Calibrations' folder should be added to the application ouput folder in order to measure the impedance

            #   Deskew signals at the DUT level. DUT should be removed before calling this function to get DUT starting point.
            #   This step is required in order to measure the impedance profile.
            #   It can be done once for same setup and offsets should be saved for consecutive measurements.
            print("Calibrate TDR")
            ok = input('Please remove your DUT then press enter to continue...')
            offset = MLTDR.Deskew();
            for idx in range(pymltdr.NUMRXCHANNELS):
                print("Deskew Ch %s result %s" % ( idx + 1, offset[idx]))
            #   Gets x values in ps and m
            print("Get x values")
            output = MLTDR.CalculateXValues(TimeResolution, VELOCITYFACTOR);
            xValuesps = output[0];
            xValuesm = output[1];

            repeat = True

            while repeat:
                #   Checks DUT end point. This step can be skipped, it is used to measure the DUT length.
                print("Measure DUT length...")
                ok = input('Please connect your DUT and leave the second end open then press enter to continue...')
                offset2 = MLTDR.Deskew();
                for idx in range(pymltdr.NUMRXCHANNELS):
                    print("Ch %s DUT length: %.2f ps , %.2f m" % ( idx + 1, xValuesps[offset2[idx] - offset[idx]], xValuesm[offset2[idx]-offset[idx]] ))
                
                #   Reads TDR values
                params = pymltdr.mltdrImpedanceParams(pymltdr.MLTDRINPUT.DIFFERENTIAL, offset, AVGTYPE, NUMAVERAGES, SMOOTHINGWINDOWSIZE, IMPEDANCEOFFSET);
                print("Read impedance values")
                output = MLTDR.ReadTDRValues(params, TimeResolution);
                tdr = output[0];
                tdrOhms = output[1];

                #   Checks impedance mask results
                print("Mask results:")
                for idx, y in enumerate(tdrOhms):
                    if len(tdrOhms[idx]) > 0:
                        try:
                            xValues = list(xValuesps)
                            maskRange = xValues[offset2[idx]] - xValues[offset[idx]]
                            minX = TDRMASK_MINX * maskRange / 100
                            maxX = TDRMASK_MAXX * maskRange / 100
                            startingPoint = xValues.index(next(x for x in xValues if x >= minX))
                            endPoint = xValues.index(next(x for x in xValues if x > maxX))
                            status = "Passed"
                            for i in range(startingPoint, endPoint):
                                if ((y[i] < TDRMASK_MINY) or (y[i] > TDRMASK_MAXY)):
                                    status = "Failed"
                                    break
                            print("Channel %i: %s" % (idx + 1, status))
                        except StopIteration:
                            print("Could not check mask on Channel %i" % (idx + 1))

                #   Saves impedance values for first channel
                if SAVECSV:
                    print("Save impedance to " + TDRDATAOUTPUT)
                    with open(TDRDATAOUTPUT, "w") as fio:
                        fio.write("{},{},{},{},{}\n".format("Time (ps)", "Ch1 (Ohm)", "Ch2 (Ohm)", "Ch3 (Ohm)", "Ch4 (Ohm)"))
                        for x, y1, y2, y3, y4 in zip(xValuesps, tdrOhms[0], tdrOhms[1], tdrOhms[2], tdrOhms[3],):
                            fio.write("{},{},{},{},{}\n".format(x, y1, y2, y3, y4))
                
                #   Plots impedance values of all channels
                fig, a = plt.subplots(2,2)
                for idx, ch in enumerate(tdrOhms):
                    if len(ch) > 0:
                        row = 0 if idx < 2 else 1
                        col = 0 if idx % 2 == 0 else 1

                        a[row][col].plot(xValuesps[0:len(ch)], ch)
                        if math.inf in ch:
                            a[row][col].set_xlim(xValuesps[0], xValuesps[ch.index(math.inf) + 100])
                        a[row][col].set_title('Ch' + str(idx + 1))
                        a[row][col].set_xlabel('Time (ps)')
                        a[row][col].set_ylabel('Impedance Profile (Ohm)')
                        a[row][col].grid()
                
                plt.tight_layout()
                plt.savefig('TDRPlots.png')
                plt.show()

                #   Checks if user wants to test another DUT
                ok = input("Test Done. Would you like to test another DUT? [y/n]")
                while (ok.lower().strip() != 'y' and ok.lower().strip() != 'n'):
                    ok = input("Please enter 'y' or 'n' (yes/no): ")

                repeat = ok.lower().strip() == 'y'
        #############################################################################
        
    #############################################################################
    except KeyboardInterrupt:
        print("Interrupted by user!")
        sys.exit(131)
    #############################################################################
if __name__ == "__main__":
    main()
