#pragma once

#ifndef ML_TDR_API_H__
#define ML_TDR_API_H__

#ifndef MLTDRAPICALL
#ifdef __cplusplus
#   ifdef WIN32
#       ifndef ML_IMPORTS_API
#           define MLTDRAPICALL extern "C" __declspec(dllexport)
#       else
#           define MLTDRAPICALL extern "C" __declspec(dllimport)
#       endif
#   else
#       define MLTDRAPICALL extern "C" 
#   endif // WIN32
#else
#   define MLTDRAPICALL
#endif
#endif

#ifndef MLTDRSTACKMODE
#   ifdef WIN32
#       define MLTDRSTACKMODE   __cdecl
#   else
#       define MLTDRSTACKMODE
#   endif // WIN32
#endif // !MLTDRSTACKMODE

#ifdef __cplusplus
#include <complex>
#endif

#define MLMAXCHANNELS 8                 // Maximum number of channels in a device.
#define NUMTXCHANNELS 4                 // Number of TX channels.
#define MLBERTTAPSNBR 7                 // BERT Number of Taps.
#define CAL_VAR_SIZE 7                  // Number of calibration variables.
#define CAPTUREOUTPUTSIZE 11            // Capture results array size.
#define MAX_ADD_LENGTH      256         // Maximum address length


typedef enum { MLTDRAPIVERSION = 13 };  // Incremental module version


typedef unsigned long long  ulong;      // 64-bit unsigned type
typedef unsigned short      ushort;     // Unsigned 16-bit (short) integer type
typedef unsigned int        uint;       // Unsigned 32-bit (int) integer type
typedef unsigned long long  ulong;      // Unsigned 64-bit (long) integer type
#ifndef __cplusplus
typedef int bool;
#endif


/// <summary>
/// API wrapper instance type.
/// </summary>
typedef struct  mltdrapi mltdrapi;


/// <summary>Create API instance</summary>
/// <returns>Created instance</returns>
MLTDRAPICALL mltdrapi* MLTDRSTACKMODE mltdr_createInstance();

/// <summary>Destroy API instance</summary>
/// <param name="inst">Pointer of instance to destroy</param>
MLTDRAPICALL void MLTDRSTACKMODE mltdr_destroyInstance(mltdrapi* inst);

/// <summary>
/// Complex number
/// </summary>
typedef struct
{
    double Real;        // Real part
    double Imaginary;   // Imaginary part
}Complex;

/// <summary>
/// Connection type.
/// </summary>
typedef enum
{
    UNDETECTED = -1,    // Detection failed
    ETH2MICRO = 0,      // Ethernet connection to microcontroller
    ETH2FPGA = 1,       // Ethernet connection to FPGA
    PXI2FPGA = 2,       // PCIe connection to FPGA
}MLTDRCONNMODE;


/// <summary>
/// Detect connection type.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="address">IP address or PXI resource name</param>
/// <returns>Detected connection type</returns>
MLTDRAPICALL MLTDRCONNMODE MLTDRSTACKMODE mltdr_detectConnectivity(mltdrapi* inst, char * address);

/// <summary>
/// Pulse mode
/// </summary>
typedef enum
{
    BERT_MODE,          //  BERT mode only     
    SPARAM_MODE,        //  Saprameter measurement 
    BERT_SCOPE_MODE,    //  BERT and Scope mode (eye/pattern/THD measurments)
    TDR_MODE,           //  TDR mode (not implemented yet)
    SCOPE_MODE          //  Scope mode only (using external bert) (not implemented yet) 
}MLTDRPULSE_MODE;

/// <summary>
/// Pattern length
/// </summary>
typedef enum
{
    PRBS7,
    PRBS9,
    PRBS15,
    PRBS23,
    PRBS31,
    USERDEFINED,
    PRBS11,
    PRBS13,
}MLTDRPattern;

/// <summary>
/// Signal coding
/// </summary>
typedef enum {
    SC_NRZ = 0,
    SC_PAM4 = 1,
}MLTDRSIGNALCODING;

/// <summary>
/// THD Mode.
/// </summary>
typedef enum
{
    MLTDRTHDMODE_oddHarmonics = 0,
    MLTDRTHDMODE_evenHarmonics,
    MLTDRTHDMODE_oddAndEvenHarmonics
}MLTDRTHDMODE;

typedef enum
{
    Hardware,
    Software
}MLTDRAVERAGINGTYPE;

typedef enum RX_DFE_Mode {
    RX_MODE_SLC1,
    POR_DSP_MODE_SLC1_RC_SLC2,
    POR_DSP_MODE_SLC1_RC_LDEQ,
    POR_DSP_MODE_DFE1,
    POR_DSP_MODE_DFE1_RC_DFE2
};

typedef enum Pulse_Pattern_Divider {
    Div_32 = 32,
    Div_64 = 64,
    Div_128 = 128,
    Div_320 = 320,
    Div_640 = 640,
    Div_1280 = 1280,
    Div_2560 = 2560,
    Div_5120 = 5120
};

typedef enum
{
    DIFFERENTIAL,   // Differential measurement
    SINGLEENDED,    // Single-Ended measurement
}MLTDRINPUT;

typedef struct
{
    MLTDRINPUT input;           // Signal input measurement
    int Offset[MAX_CHANNELS];   // Deskew offset array calculated usign mltdr_Deskew function
    MLTDRAVERAGINGTYPE type;    // Averaging type
    int nAvgs;                  // Number of averages (Default 1: No avg)
    int smoothingWindow;        // Smoothing window size (Default 0: No smoothing)
    double impedanceOffset[MAX_CHANNELS]; // User defined impedance offset
}mltdrImpedanceParams;

typedef struct APIPresetStruct
{
    MLTDRPULSE_MODE configMode;
    int Mode;
    double LineRate;
    // Bert Configuration
    bool GrayMapping;
    bool Precoding;
    bool VGA_Tracking;
    bool FEC_Enable;
    int FEC_Mode;
    RX_DFE_Mode DFE_Mode[4];
    bool IEEE;
    bool InternalClock; // internal = 1 ; external = 0;
    double ExternalClockValue;  //  Only if Internal Clock is false
    int OutputClock;
    int ClockDivider;
    MLTDRPattern BertTxPattern[4];
    MLTDRPattern BertRxPattern[4];
    bool RxInvert[4];
    int Scaling[4];
    int InnerEye;
    int OuterEye;
    int MainTap[4];
    int PreEmphasis[4];
    int PostEmphais[4];
    bool BER_Enable;
    bool Enable7Taps;
    int Tap0[4];
    int Tap1[4];
    int Tap2[4];
    int Tap3[4];
    int Tap4[4];
    int Tap5[4];
    int Tap6[4];
    ulong UserPattern1;
    ulong UserPattern2;
    ushort Repetition1;
    ushort Repetition2;
    // DSO Configuration
    int NumberOfSamples;
    bool DSO_TDR;
    bool ResetChip;
    double SamplingFrequency;
    int DSOPatternLength;
    // TDR Confuguration
    Pulse_Pattern_Divider PulseDivider;
    bool IsUsingTiClock;
    ushort DSO_ClockDivider;  //  TODO : Accomodate high clock inputs (Over 500 MHz)
    // For Silab
    int SilabCount;

}APIPresetStruct;


/// <summary>
/// Configuration parameters.
/// </summary>
typedef struct
{
    MLTDRPULSE_MODE PulseMode;                                  // Device pulse mode
    int ChannelsConfig[MLMAXCHANNELS];                          // Channels config array(size of MAX_CHANNELS = 8) :
                                                                // Values specify channels state and indexes indicate channels number. 
                                                                // PS: 4035 channels labeled 'Tx' have indexes 0 to 3 ; Channels labeled 'CH' 4 to 7)
                                                                // value 0: unused channel;
                                                                // Pulse mode TDR:  Value > 0 : TDR active channel else channel off;
                                                                // Pulse mode TDT: Value v1 > 0 : Tx channel and Value v2 < 0 : Rx Channel
                                                                // Each Tx may have one or more rx channels that are identified by v2=-v1;
    int SamplesCount;                                           // Number of samples 
    double LineRate;                                            // TDT Line rate
    int NumberOfAverages;                                       // Number of averages
    MLTDRPattern TxPattern[NUMTXCHANNELS];                      // TDT Pattern Length
    MLTDRSIGNALCODING SignalCoding;                             // TDT signal Coding
    double SamplingFrequency;                                   // Desired sampling frequency; Default value is NAN (Use API generated value).
    int MainTap[NUMTXCHANNELS];                                 // BERT TX Channels MainTap array (array index = Tx Channel index and value = Main Tap for each Tx)
                                                                // Values are between -1000 and 1000.
    int PreEmphasis[NUMTXCHANNELS];                             // BERT TX Channels Pre-Emphasis array (array index = Tx Channel index and value = Pre-Emphasis for each Tx)
                                                                // Values are between -1000 and 1000.
    int PostEmphasis[NUMTXCHANNELS];                            // BERT TX Channels Post-Emphasis array (array index = Tx Channel index and value = Post-Emphasis for each Tx)
                                                                // Values are between -1000 and 1000.
    int OutputLevel[NUMTXCHANNELS];                             // BERT TX Channels output level array (array index = Tx Channel index and value = output level for each Tx)
                                                                // Values are between 60 and 120.
    int OuterEyeLevel[NUMTXCHANNELS];                           // BERT TX Channels Pam4 outer eye level array (array index = Tx Channel index and value = outer eye level for each Tx)
                                                                // Values are between 1500 and 2000.
    int InnerEyeLevel[NUMTXCHANNELS];                           // BERT TX Channels Pam4 inner eye level array (array index = Tx Channel index and value = inner eye level for each Tx)
                                                                // Values are between 500 and 1500.

    unsigned long long UserDefinedPatternLow[NUMTXCHANNELS];    // BERT Tx Channels user defined LSB (16 hex numbers -> 64bits)
    unsigned long long UserDefinedPatternMid[NUMTXCHANNELS];    // BERT Tx Channels user defined MSB used in PAM4 signal coding only (16 hex numbers -> 64bits)
    ushort repetitionLow[NUMTXCHANNELS];                        // BERT Tx Channels UserDefinedPatternLow number of repetitions. (range 1->255)
    ushort repetitionMid[NUMTXCHANNELS];                        // BERT Tx Channels UserDefinedPatternMid number of repetitions. (range 1->255)
    bool enable7Taps;                                           // Enables BERT Tx Channels 7 taps
    bool enableGrayMapping;                                     // Enables PAM4 Gray Mapping
    double stdDeviceTotalNoiseCalibration;                      // Scope total noise
    MLTDRAVERAGINGTYPE averagingType;							// Averaging type
    int presetIndex;							                // Enabled FW presets if > 0, disable otherwise

}mltdrConfigurationParams;

/// <summary>
/// Configure device 
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="confparams">Configuration parameters </param>
/// <param name="result">
/// Configure results array.
/// 
/// Current implementation expects an array size of 4 mapped:
///
/// Time Base, Frequency Resolution, Time Resolution, Sampling Frequency
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_configure(mltdrapi* inst, mltdrConfigurationParams confparams, double* result);

/// <summary>
/// TDT Calibration (get insertion loss reference circuit).
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="SparamRef">
///2D output array (MLMAXCHANNELS * Time Resolution):
///First dimension defines the channel index.
///Second dimension defines the reference array of each channel.
///</param>
/// <param name="RefSize">Size of desired captured reference of a channel</param>
/// <returns>
/// Captured reference size.
/// </returns>
MLTDRAPICALL int MLTDRSTACKMODE mltdr_getSparameterReference(mltdrapi* inst, double* sparamRef, int RefSize);


/// <summary>
/// Sparam configuration parameters.
/// </summary>
typedef struct
{
    int SmoothingAverageWindowSize; // Smoothing window size
    int FrequencyResolution;        // Resolution = Pattern Length
    int OffsetDB;                   // Offset in dB
}mltdrSparamConfig;


#ifdef __cplusplus
/// <summary>
/// Calculate S21 Measurement.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="SparamRef">
///2D input array (MLMAXCHANNELS * RefSize):
///First dimension defines the channel index.
///Second dimension defines the reference array of each channel.
///</param>
/// <param name="RefSize">Size of desired captured reference of a channel</param>
/// <param name="SparamConfig">SparamConfig parameters</param>
/// <param name="frequencyValues">Frequency output array of frequency resolution size</param>
/// <param name="S21Values">
///S21 complex 2D output array (MLMAXCHANNELS * Frequency Resolution):
///First dimension defines the channel index.
///Second dimension defines the s21 complex array of each channel.
///</param>
/// <returns>
/// S21 samples count.
/// </returns>
MLTDRAPICALL int MLTDRSTACKMODE mltdr_calculateSdd21(mltdrapi* inst, double*SparamRef, int RefSize, mltdrSparamConfig SparamConfig,
    double* frequencyValues, std::complex<double>* S21Values);
#endif

/// <summary>
/// Calculate S21 Measurement.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="SparamRef">
///2D input array (MLMAXCHANNELS * RefSize):
///First dimension defines the channel index.
///Second dimension defines the reference array of each channel.
///</param>
/// <param name="RefSize">Size of desired captured reference of a channel</param>
/// <param name="SparamConfig">SparamConfig parameters</param>
/// <param name="frequencyValues">Frequency output array of frequency resolution size</param>
/// <param name="S21Values">
///S21 complex 2D output array (MLMAXCHANNELS * Frequency Resolution):
///First dimension defines the channel index.
///Second dimension defines the s21 complex array of each channel.
///</param>
/// <returns>
/// S21 samples count.
/// </returns>
/// <remarks>
/// The only difference from 'mltdr_calculateSdd21' function is that the Complex type used in this function is a local defined structure.
/// </remarks>
MLTDRAPICALL int MLTDRSTACKMODE mltdr_calculateSdd21_C(mltdrapi* inst, double*SparamRef, int RefSize, mltdrSparamConfig SparamConfig,
    double* frequencyValues, Complex* S21Values);


#ifdef __cplusplus
/// <summary>
/// DeEmbed S21 File.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="freq">Frequency input array of measured s21</param>
/// <param name="S21Values">S21 complex input array of measured s21</param>
/// <param name="freqSize">s21 freq and values size of measured s21</param>
/// <param name="freq_DeEmbedFile">Frequency input array of the trace to de-embed</param>
/// <param name="S21Values_DeEmbedFile">S21 complex input array of the trace to de-embed</param>
/// <param name="freq_DeEmbedFileSize">s21 freq and values size of the trace to de-embed</param>
/// <param name="S21Values_Output">s21 complex output array</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_DeEmbedSparam(mltdrapi* inst, double* freq, std::complex<double>* S21Values, int freqSize,
    double* freq_DeEmbedFile, std::complex<double>* S21Values_DeEmbedFile, int freq_DeEmbedFileSize, std::complex<double>* S21Values_Output);
#endif

/// <summary>
/// DeEmbed S21 File.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="freq">Frequency input array of measured s21</param>
/// <param name="S21Values">S21 complex input array of measured s21</param>
/// <param name="freqSize">s21 freq and values size of measured s21</param>
/// <param name="freq_DeEmbedFile">Frequency input array of the trace to de-embed</param>
/// <param name="S21Values_DeEmbedFile">S21 complex input array of the trace to de-embed</param>
/// <param name="freq_DeEmbedFileSize">s21 freq and values size of the trace to de-embed</param>
/// <param name="S21Values_Output">s21 complex output array</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
/// <remarks>
/// The only difference from 'mltdr_DeEmbedSparam' function is that the Complex type used in this function is a local defined structure.
/// </remarks>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_DeEmbedSparam_C(mltdrapi* inst, double* freq, Complex* S21Values, int freqSize,
    double* freq_DeEmbedFile, Complex* S21Values_DeEmbedFile, int freq_DeEmbedFileSize, Complex* S21Values_Output);

/// <summary>
/// Captures and aligns calibrated samples for all channels.
/// </summary>
/// <param name="inst">Instance number.</param>
/// <param name="samplesNumber">Capture size.</param>
/// <param name="xPattern">
/// Pointer to output pattern X-data array.
/// Array must be properly allocated for specified number of samples for all channels.
/// </param>
/// <param name="xEye">
/// Pointer to output eye X-data array.
/// Array must be properly allocated for specified number of samples for all channels
/// (2 x number of samples).
/// </param>
/// <param name="ScopeSamples">
/// Pointer to output data container array.
/// Array must be properly allocated for specified number of samples for all channels
/// (2 x number of samples).
/// Data from each channel are linearised sequentially in consecutive blocks of N samples.
/// (i.e. compatible with C#'s [,] array).
/// </param>
/// <param name="lockOutput">
/// Pointer to output pattern lock output array.
/// Array must be allocated for device's number of channels.
/// </param>
/// <param name="results">
/// Pointer to measurement results container array.
/// Array must be properly allocated for 11 elements for all channels:
/// Array output is mapped:
/// Min, Max, PeakToPeak, CrossingX, CrossingY, Top, base, LevelZero, LevelOne, LevelTwo, LevelThree
/// Data from each channel are linearised sequentially in consecutive blocks of 11 elements.
/// (i.e. compatible with C#'s [,] array).
/// </param>
/// <returns>Operation success.</returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_CaptureData(mltdrapi* inst, int samplesNumber, double* xPatternAll, double* xEyeAll,
    double* ScopeSamples, bool* lockOutput, double* results);

/// <summary>
/// Set Tx channels 7 taps.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="channels">Tx channels input array</param>
/// <param name="nChannels">Tx channels input array size</param>
/// <param name="Taps">
/// Tx channels Tap values 2D input array (nChannels * MLBERTTAPSNBR):
/// First dimension defines index of channels array.
/// Second dimension defines taps array of each channel (MLBERTTAPSNBR taps per channel).
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTx7Taps(mltdrapi* inst, int* channels, int nChannels, int* Taps);

/// Get Rx FFE taps.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="channel">Tx channel input</param>
/// <param name="taps">
/// Rx FFE taps 1D input array (should accomodate 15 taps)
///</param>
/// <returns>
/// Function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetFFE(mltdrapi* inst, int channel, double* taps);

/// <summary>
/// Calculates THD.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="nSamples">Number of samples.</param>
/// <param name="xPattern">Pattern X data.</param>
/// <param name="yValues">Eye Y data.</param>
/// <param name="UserDefinedPatternLow">User defined pattern generated.</param>
/// <param name="lastHarmonicNumber">number of the last harmonic to be included in THD calculation</param>
/// <param name="thdMode">3  modes:odd harmonics-even harmonics- odd and even harmonic  (to be included in calculations)</param>
/// <returns>
/// THD measurment double value.
/// </returns>
MLTDRAPICALL double MLTDRSTACKMODE mltdr_GetTHD(mltdrapi* inst, int samplesNumber, double* xPattern, double* ScopeSamples, ulong UserDefinedPatternLow, int lastHarmonicNumber, MLTDRTHDMODE thdMode);

/// <summary>
/// Start realtime BER capture.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_StartRealtimeBER(mltdrapi* inst);

/// <summary>
/// Stop realtime BER capture.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_StopRealtimeBER(mltdrapi* inst);

/// <summary>
/// Read current realtime BER.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="time_in_ms">Current BER timer value in milliseconds.</param>
/// <param name="errorCounts">Real-time error counts array for all channels.</param>
/// <param name="BERs">Calculated real-time BER output array for all-channels.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ReadRealtimeBER(mltdrapi* inst, uint time_in_ms[], ulong errorCounts[], double BERs[]);

/// <summary>
/// Read current MSB and LSB error counts in PAM4 mode.
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="errorCountsMSB">MSB error counts array for all channels.</param>
/// <param name="errorCountsLSB">LSB error counts array for all channels.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ReadBerMSBLSB(mltdrapi* inst, ulong errorCountsMSB[], ulong errorCountsLSB[]);

/// <summary>
/// Checks Channel's PRBS lock.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="channel">Channel index. </param>
/// <returns>
/// True if channel is locked, False otherwise.
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_PRBSVerfication(mltdrapi* inst, int channel);

/// <summary>
/// Checks Tx Channel's lock.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="channel">Channel index. </param>
/// <param name="status">True if channel is locked, False otherwise.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ReadTXLock(mltdrapi* inst, int channel, bool *status);

/// <summary>
/// Calculates PAM4 RLM.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="nSamples">Number of samples.</param>
/// <param name="xPattern">Pattern X data.</param>
/// <param name="xEye">Eye X data.</param>
/// <param name="yValues">Eye Y data.</param>
/// <param name="initializeData">update data.</param>
/// <param name="eyeLevels">
/// PAM4 eye levels in the following order:
/// currentZeroLevel , currentOneLevel, currentTwoLevel ,currentThreeLevel.
/// <param>
/// <param name="output">
/// Output array of 2 elements containing in the following order:
/// currentRLM_120D (IEEE 802.3 Annex 120D), currentRLM_94(IEEE 802.3 clause 94).
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetPAM4RLM(mltdrapi* inst, int nSamples, double* xPattern, double* xEye, double* ScopeSamples, bool initializeData, double* eyeLevels, double* output);

/// <summary>
/// Calculates PAM4 RLM (current data).
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="output">
/// Output array of 2 elements containing in the following order:
/// currentRLM_120D (IEEE 802.3 Annex 120D), currentRLM_94(IEEE 802.3 clause 94).
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetCurrentPAM4RLM(mltdrapi* inst, double* output);

#ifdef __cplusplus
/// <summary>
/// Calculates 3 dB BW.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="Fin_Hz">Frequency input array of measured s21.</param>
/// <param name="Sin">S21 complex input array of measured s21.</param>
/// <param name="IsAbsolute3db">true: uses absolute 3dB value, false: uses rolloff from 'StartFrequencyInHz'.</param>
/// <param name="StartFrequencyInHz">BW Start Frequency if 'IsAbsolute3db' is false</param>
/// <returns>
/// 3 db BW value in Hz.
/// </returns>
MLTDRAPICALL double MLTDRSTACKMODE mltdr_Get3dBBandwidth(mltdrapi* inst, double* Fin_Hz, int size, std::complex<double>* Sin, bool IsAbsolute3db, double StartFrequencyInHz);
#endif

/// <summary>
/// Calculates 3 dB BW.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="Fin_Hz">Frequency input array of measured s21.</param>
/// <param name="Sin">S21 complex input array of measured s21.</param>
/// <param name="IsAbsolute3db">true: uses absolute 3dB value, false: uses rolloff from 'StartFrequencyInHz'.</param>
/// <param name="StartFrequencyInHz">BW Start Frequency if 'IsAbsolute3db' is false.</param>
/// <returns>
/// 3 db BW value in Hz.
/// </returns>
/// <remarks>
/// The only difference from 'mltdr_Get3dBBandwidth' function is that the Complex type used in this function is a local defined structure.
/// </remarks>
MLTDRAPICALL double MLTDRSTACKMODE mltdr_Get3dBBandwidth_C(mltdrapi* inst, double* Fin_Hz, int size, Complex* Sin, bool IsAbsolute3db, double StartFrequencyInHz);

/// <summary>
/// Sets Tx on/off.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="channels">channels input array.</param>
/// <param name="nChannels">size of channels array.</param>
/// <param name="status">true: Sets Tx off; false: Sets Tx on.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxOff(mltdrapi* inst, int* channels, int nChannels, bool status);

/// <summary>
/// Get Noise Profile.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="xEye">eye x values</param>
/// <param name="yValues">eye y values</param>
/// <param name="xPattern">pattern x values</param>
/// <param name="samplesCount">number of samples</param>
/// <param name="isInitialized">Pre-initialization flag.</param>
/// <param name="result">
/// Output array of 2 elements containing respectively: Deterministic Noise, Random Noise RMS.
/// </param>
/// <returns>Operation success.</returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetNoiseProfile(
    mltdrapi* inst, double* xEye, double* yValues, double* xPattern, int samplesCount,
    bool isInitialized, double result[]);

/// <summary>
/// Measure Scope Vrms.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="xPattern">pattern x values.</param>
/// <param name="yValues">eye y values.</param>
/// <param name="result">Output scope Vrms value.</param>
/// <returns>Operation success.</returns>
/// <remarks>
/// Scope Vrms is the device total noise if no input signal is added.
/// </remarks>

MLTDRAPICALL bool MLTDRSTACKMODE mltdr_MeasureScopeVrms(mltdrapi* inst, double * xPattern, double * yValues, int samplesCount, double * result);

/// <summary>
/// Get Vrms value.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="xPattern">pattern x values.</param>
/// <param name="yValues">eye y values.</param>
/// <param name="result">Output Vrms value</param>
/// <returns>Operation success.</returns>
/// <remarks>
/// requires mltdr_MeasureScopeVrms in order to measure device noise that will be deEmbedded from Vrms resulted value.
/// </remarks>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetVrms(mltdrapi* inst, double * xPattern, double * yValues, int samplesCount, double * result);

/// <summary>
/// Deskew signals at the DUT level.
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="offset">Deskew offset output array of size MLMAXCHANNELS</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
/// <remarks>
/// 'TDR_Calibrations' folder should be added to the application ouput folder in order to measure the impedance profile
/// DUT should be removed before calling this function. Deskew is done to precisely align the signals at the DUT.
/// </remarks>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_Deskew(mltdrapi* inst, int* offset);

/// <summary>
/// Calculates x values
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="resolution">Time Resolution</param>
/// <param name="velocityFactor"> Velocity factor: The ratio of actual speed to the speed of light -> between 0 and 1</param>
/// <param name="xValuesPs">X values output array in ps of size TimeResolution (96377 by default)</param>
/// <param name="xValuesPs">X values output array in m of size TimeResolution (96377 by default) calculated using the velocityFactor input</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_CalculateXValues(mltdrapi* inst, int resolution, double velocityFactor, double* xValuesPs, double* xValuesm);

/// <summary>
/// Read tdr Values for all channels
/// </summary>
/// <param name="inst">Instance pointer</parm>
/// <param name="params">MLTDRPARAMS struct parameters</param>
/// <param name="tdr">
/// Normalized Impedance values 2D output array (MAX_CHANNELS * TimeResolution (96377 by default)):
/// First dimension defines index of channels array.
/// Second dimension defines channel impedance array.
///</param>
/// <param name="tdrOhm">
/// Ohm Impedance values 2D output array (MAX_CHANNELS * TimeResolution (96377 by default)):
/// First dimension defines index of channels array.
/// Second dimension defines channel impedance array in ohms.
///</param>
/// <param name="datasize">
/// Number of samples array of size MAX_CHANNELS
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// <remarks>
/// 'TDR_Calibrations' folder should be added to the application ouput folder in order to measure the impedance profile
/// requires mltdr_Deskew in order to precisely align the signals at the DUT.
/// </remarks>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ReadTDRValues(mltdrapi* inst, mltdrImpedanceParams params, double* tdr, double* tdrOhm, int* datasize);

#ifdef MACOM
/// <summary>
/// Configure device using FW preset functions
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="confparams">Configuration parameters </param>
/// <param name="index">index of FW-PRESET configuration </param>
/// <param name="samplingfrequency">real sampling frequncy corresponding to the preset index </param>
/// <param name="result">
/// Configure results array.
/// 
/// Current implementation expects an array size of 2 mapped:
///
/// Time Base, Frequency Resolution
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_configure_preset(mltdrapi* inst, int preset_index, double *result, mltdrConfigurationParams confparams); // Chroma implementation

/// <summary>
/// Set Instrument in preset mode
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="value">Enable/Disable preset mode.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetPresetMode(mltdrapi* inst, bool value);

/// <summary>
/// Read Calibration Factors from the instrument and save to file
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_UpdateCalFactors(mltdrapi* inst);

/// <summary>
/// Captures raw samples for all channels.
/// </summary>
/// <param name="inst">Instance number.</param>
/// <param name="samplesNumber">Capture size.</param>
/// <param name="xPattern">
/// Pointer to output pattern X-data array.
/// Array must be properly allocated for specified number of samples for all channels.
/// </param>
/// <param name="xEye">
/// Pointer to output eye X-data array.
/// Array must be properly allocated for specified number of samples for all channels
/// (2 x number of samples).
/// </param>
/// <param name="ScopeSamples">
/// Pointer to output data container array.
/// Array must be properly allocated for specified number of samples for all channels
/// (2 x number of samples).
/// Data from each channel are linearised sequentially in consecutive blocks of N samples.
/// (i.e. compatible with C#'s [,] array).
/// </param>
/// <param name="lockOutput">
/// Pointer to output pattern lock output array.
/// Array must be allocated for device's number of channels.
/// </param>
/// <returns>Operation success.</returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_GetRawData(mltdrapi* inst, int samplesNumber, double* xPatternAll, double* xEyeAll,
    double* ScopeSamples, bool* lockOutput);

/// <summary>
/// Sets Tx on/off. Remove Delay between channel based calls
/// </summary>
/// <param name="inst">Instance pointer.</param>
/// <param name="channels">channels input array.</param>
/// <param name="nChannels">size of channels array.</param>
/// <param name="status">true: Sets Tx off; false: Sets Tx on.</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxOffNoDelay(mltdrapi* inst, int* channels, int nChannels, bool status);

/// <summary>
/// Set Tx channels Main Tap Value.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="channels">Tx channels input array</param>
/// <param name="nChannels">Tx channels input array size</param>
/// <param name="values">
/// Tx channels Main Tap values input array (nChannels):
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxMainTap(mltdrapi* inst, int* channels, int nChannels, int* values);

/// <summary>
/// Set Tx channels pre-Emphasis.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="channels">Tx channels input array</param>
/// <param name="nChannels">Tx channels input array size</param>
/// <param name="values">
/// Tx channels pre-Emphasis values input array (nChannels):
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxpreEmphasis(mltdrapi* inst, int* channels, int nChannels, int* values);

/// <summary>
/// Set Tx channels postEmphasis.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="channels">Tx channels input array</param>
/// <param name="nChannels">Tx channels input array size</param>
/// <param name="values">
/// Tx channels post-Emphasis values input array (nChannels):
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxpostEmphasis(mltdrapi* inst, int* channels, int nChannels, int* values);

/// <summary>
/// Set Tx channels Pattern.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="channels">Tx channels input array</param>
/// <param name="nChannels">Tx channels input array size</param>
/// <param name="txpattern">
/// Tx channels pattern type:
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_SetTxpattern(mltdrapi* inst, int* channels, int nChannels, MLTDRPattern txpattern);
#endif

/// <summary>
/// Initialize instances.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="saveConfig">Config application path</param>
/// <param name="indexes">Preset indexes array</param>
/// <param name="size">Number of presets. Maximum is 15</param>
/// <returns>
/// function success (true if successful, false otherwise).
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_InitializeInstance(mltdrapi* inst, char saveConfig[MAX_ADD_LENGTH], int* indexes, int size);

/// <summary>
/// Switches module to use Preset Configuration Mode
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_EnablePreset(mltdrapi* inst);

/// <summary>
/// Loads all the previosuly saved preset configuraations from EEPROM.
/// This API call is called only one time.
/// Preset Configuration should be enabled.
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_LoadPreset(mltdrapi* inst, int index);

/// <summary>
/// Applies one of the preset configuration
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="index">
/// Index of the configuration to be used.
/// Must be between 0 and 14
///</param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ApplyPreset(mltdrapi* inst, int index);

/// <summary>
/// Loads one of the previously saved preset configurations
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="presetIndex">
/// Index of the configuration to be read.
/// Must be between 0 and 14
/// </param>
/// <param name="presetConfiguration">
/// Struct that contains neccessary data about the preset configurations.
/// Returned by reference
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ReadPresetConfiguration(mltdrapi* inst, int presetIndex, APIPresetStruct *presetConfiguration);

/// <summary>
/// Writes one preset configurations to a desired index
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="presetIndex">
/// Index of the configuration to be saved.
/// Must be between 0 and 9
/// </param>
/// <param name="presetConfiguration">
/// Struct that contains neccessary configurations for one preset.
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_WritePresetConfiguration(mltdrapi* inst, int presetIndex, APIPresetStruct presetConfiguration);

#ifdef PRV_PRESET

/// <summary>
/// Writes one preset configurations to a desired index
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="presetIndex">
/// Index of the configuration to be saved.
/// Must be between 0 and 14
/// </param>
/// <param name="presetConfiguration">
/// Struct that contains neccessary configurations for one preset.
/// </param>
/// <returns>
/// function success (true if successful, false otherwise).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_WritePrivilegedPresetConfiguration(mltdrapi* inst, int presetIndex, APIPresetStruct presetConfiguration);

#endif

/// <summary>
/// Loads all preset configurations from a specific file and saves them in EEPROM
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="path">
/// Full path directory with the file name and its extension as follows: full_path/full_name.extension
/// </param>
/// <returns>
/// function success (true if successful, false if failed to open file or connection failure).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_LoadPresetConfigurationFromFile(mltdrapi* inst, const char * path);

/// <summary>
/// Read all preset configurations from EEPROM and saves them in a specific file
/// </summary>
/// <param name="inst">Instance pointer</param>
/// <param name="path">
/// Full path directory without the file name.
// FIle will be save in full_path/ML-PresetConfiguration.txt
/// </param>
/// <returns>
/// function success (true if successful, false if failed to open file or connection failure).
/// </returns>
MLTDRAPICALL bool MLTDRSTACKMODE mltdr_ExportPresetConfigurationToFile(mltdrapi* inst, const char * path);

#endif // ML_TDR_API_H__
