"""ML-TDR API Python wrapper."""

import sys
import os
import ctypes
import enum

__all__ = ('ML_TDR_TDT_API',)
__version__ = "6.1.2"

DLLPATH32 = r"x86\ML_TDR_TDT_API.dll"
"""str: 32-bit DLL path."""

DLLPATH64 = r"x64\ML_TDR_TDT_API.dll"
"""str: 64-bit DLL path."""

MAXCHANNELS = 8
NUMTXCHANNELS = 4
NUMRXCHANNELS = 4
CAPTUREOUTPUTSIZE = 11

class MLTDRCONNMODE(enum.IntEnum):
    """Connection types enum."""
    UNDETECTED = -1  
    ETH2MICRO = 0    
    ETH2FPGA = 1     
    PXI2FPGA = 2     

class MLTDRPULSE_MODE(enum.IntEnum):
    """Pulse mode enum."""
    BERT_MODE = 0           #  BERT mode only
    SPARAM_MODE = 1         #  Saprameter measurement
    BERT_SCOPE_MODE = 2     #  BERT and Scope mode (eye/pattern/THD measurments)
    TDR_MODE = 3            #  TDR mode measurement. 
                            #  Notes:  Prefdefined settings are used (Config settings are ignored).
                            #          'TDR_Calibrations' folder should be added to the application ouput folder in order to measure the impedance.
    SCOPE_MODE = 4          #  Scope mode only (using external bert) (not implemented yet)

class MLTDRPattern(enum.IntEnum):
    """Pattern enum."""
    PRBS7 = 0
    PRBS9 = 1
    PRBS15 = 2
    PRBS23 = 3
    PRBS31 = 4
    USERDEFINED = 5
    PRBS11 = 6
    PRBS13 = 7

class MLTDRSIGNALCODING(enum.IntEnum):
    """Signal coding enum."""
    SC_NRZ = 0
    SC_PAM4 = 1

class MLTDRTHDMODE(enum.IntEnum):
    """THD mode enum."""
    MLTDRTHDMODE_oddHarmonics = 0
    MLTDRTHDMODE_evenHarmonics = 1
    MLTDRTHDMODE_oddAndEvenHarmonics = 2

class MLTDRINPUT(enum.IntEnum):
    """TDR input measurement mode."""
    DIFFERENTIAL = 0
    SINGLEENDED = 1

class MLTDRAVERAGINGTYPE(enum.IntEnum):
    """Averaging type enum."""
    Hardware = 0
    Software = 1


class mltdrImpedanceParams(ctypes.Structure):
    """Sparam configuration parameters.
        Args:
            tdrInputMode;           // Signal input measurement
            Offset[MAX_CHANNELS];   // Deskew offset array calculated usign mltdr_Deskew function
            avgType;                // Averaging type
            nAvgs;                  // Number of averages (Default 1: No avg)
            smoothingWindow;        // Smoothing window size (Default 0: No smoothing)
            impedanceOffset;        // User defined Impedance Offset
    """
    _fields_ = [("tdrInputMode",ctypes.c_int),
                ("Offset",(ctypes.c_int * MAXCHANNELS)),
                ("avgType",ctypes.c_int),
                ("nAvgs",ctypes.c_int),
                ("smoothingWindow",ctypes.c_int),
                ("impedanceOffset",(ctypes.c_double * MAXCHANNELS))]

class mltdrConfigurationParams(ctypes.Structure):

    """mltdrConfigurationParams.
        Args:
            PulseMode (int);                                // Device pulse mode
            ChannelsConfig (array of int (size 8));         // Channels config array(size of MAXCHANNELS = 8) :
                                                            // Values specify channels state and indexes indicate channels number. 
                                                            // PS: 4035 channels labeled 'Tx' have indexes 0 to 3 ; Channels labeled 'CH' 4 to 7)
                                                            // value 0: unused channel;
                                                            // Pulse mode TDR:  Value > 0 : TDR active channel else channel off;
                                                            // Pulse mode TDT: Value v1 > 0 : Tx channel and Value v2 < 0 : Rx Channel
                                                            // Each Tx may have one or more rx channels that are identified by v2=-v1;
            SamplesCount (int);                             // Number of samples 
            LineRate (double);                              // TDT Line rate
            NumberOfAverages (int);                         // Number of averages
            TxPattern (array of int(size 4);                // TDT Pattern 
            MLTDRSIGNALCODING SignalCoding(int);            // TDT signal Coding
            SamplingFrequency (double);                     // Desired sampling frequency; Default value is NAN (Use API generated value).
            MainTap (array of int(size 4));                 // TX Channels MainTap array of size MAXCHANNELS (array index = Tx Channel index and value = Main Tap for each Tx)
            PreEmphasis (array of int(size 4));             // TX Channels Pre-Emphasis array of size MAXCHANNELS (array index = Tx Channel index and value = Pre-Emphasis for each Tx)
            PostEmphasis (array of int(size 4));            // TX Channels Post-Emphasis array of size MAXCHANNELS (array index = Tx Channel index and value = Post-Emphasis for each Tx)
                                                            // If pre,post, and main values are 0 for a specific channel, API will use default configuration (MainTap=799, PreEmphasis=-153, PostEmphasis=-48)

            OutputLevel(array of int(size 4));              // BERT TX Channels output level array (array index = Tx Channel index and value = output level for each Tx)
                                                            // Values are between 60 and 120.
            OuterEyeLevel(array of int(size 4));            // BERT TX Channels Pam4 outer eye level array (array index = Tx Channel index and value = outer eye level for each Tx)
                                                            // Values are between 1500 and 2000.
            InnerEyeLevel(array of int(size 4));            // BERT TX Channels Pam4 inner eye level array (array index = Tx Channel index and value = inner eye level for each Tx)
                                                            // Values are between 500 and 1500.

            UserDefinedPatternLow(array of int(size 4));    // BERT Tx Channels user defined LSB (16 hex numbers -> 64bits)
            UserDefinedPatternMid(array of int(size 4));    // BERT Tx Channels user defined MSB used in PAM4 signal coding only (16 hex numbers -> 64bits)
            repetitionLow(array of int(size 4));            // BERT Tx Channels UserDefinedPatternLow number of repetitions. (range 1->255)
            repetitionMid(array of int(size 4));            // BERT Tx Channels UserDefinedPatternMid number of repetitions. (range 1->255)
            enable7Taps(bool);                              // Enables BERT Tx Channels 7 taps
            enableGrayMapping(bool);                        // Enables PAM4 Gray Mapping
            stdDeviceTotalNoiseCalibration(double);         // Scope total noise
            MLTDRAVERAGINGTYPE averagingType;				// Averaging type
            int presetIndex;							    // Enabled FW presets if > 0, disable otherwise
     """
    _fields_ = [("PulseMode",ctypes.c_int),
                ("ChannelsConfig",(ctypes.c_int * MAXCHANNELS)),
                ("SamplesCount",ctypes.c_int),
                ("LineRate",ctypes.c_double),
                ("NumberOfAverages",ctypes.c_int),
                ("TxPattern",ctypes.c_int * NUMTXCHANNELS),
                ("SignalCoding",ctypes.c_int),
                ("SamplingFrequency",ctypes.c_double),
                ("MainTap",(ctypes.c_int * NUMTXCHANNELS)),
                ("PreEmphasis",(ctypes.c_int * NUMTXCHANNELS)),
                ("PostEmphasis",(ctypes.c_int * NUMTXCHANNELS)),
                ("OutputLevel",(ctypes.c_int * NUMTXCHANNELS)),
                ("OuterEyeLevel",(ctypes.c_int * NUMTXCHANNELS)),
                ("InnerEyeLevel",(ctypes.c_int * NUMTXCHANNELS)),
                ("UserDefinedPatternLow",(ctypes.c_uint64 * NUMTXCHANNELS)),
                ("UserDefinedPatternMid",(ctypes.c_uint64 * NUMTXCHANNELS)),
                ("repetitionLow",(ctypes.c_uint16 * NUMTXCHANNELS)),
                ("repetitionMid",(ctypes.c_uint16 * NUMTXCHANNELS)),
                ("enable7Taps",(ctypes.c_bool)),
                ("enableGrayMapping",(ctypes.c_bool)),
                ("stdDeviceTotalNoiseCalibration", (ctypes.c_double)),
                ("averagingType", (ctypes.c_int)),
                ("presetIndex", (ctypes.c_int))]

class mltdrSparamConfig(ctypes.Structure):
    """Sparam configuration parameters.
        Args:
            SmoothingAverageWindowSize; // Smoothing window size
            FrequencyResolution;        // Resolution = Pattern Length
            OffsetDB;                   // Offset in dB
    """
    _fields_ = [("SmoothingAverageWindowSize",ctypes.c_int),
                ("FrequencyResolution",ctypes.c_int),
                ("OffsetDB",ctypes.c_int)]

class complex(ctypes.Structure):
    """Complex number.
        Args:
            Real;           // real part
            Imaginary;      // imaginary part
    """
    _fields_ = [("Real",ctypes.c_double),
                ("Imaginary",ctypes.c_double)]

class _IntPtr(ctypes.Structure):
    """Temporary structure for IntPtr creation."""
    pass

class IntPtr(ctypes.POINTER(_IntPtr)):
    """Structure pointer used as class pointer."""
    pass

class MLTDRAPI():
    """Ml-TDR API Wrapper class."""

    def __init__(self):
        """Create new Ml-TDR API Wrapper instance.
        """
        self._Is64 = (sys.maxsize > 2 ** 32)  # Checks if in 64-bit mode
        #  Loads the appropriate DLL (raises exception on failure)
        dllPath = os.path.join(os.path.dirname(__file__), DLLPATH64 if self._Is64 else DLLPATH32)
        dlldir = os.path.dirname(dllPath)
        os.environ['PATH'] = dlldir + ";" + os.environ['PATH']
        self._api = ctypes.CDLL(os.path.basename(dllPath))
        #  Stores instance number
        self.instance = self.CreateInstance()

    def CreateInstance(self):
        """Create Ml-TDR instance.

        Returns:
            ML-TDR API instance pointer.

        """
        self._api.mltdr_createInstance.restype = IntPtr
        instance = self._api.mltdr_createInstance()
        return instance

    def InitializeInstance(self, saveConfig, indexes, size):
        """Initialize ML-TDR instance.

        Returns:
            bool: Initialization success.

        """
        self._api.mltdr_InitializeInstance.restype = ctypes.c_bool
        retVal = self._api.mltdr_InitializeInstance(self.instance, saveConfig, indexes, size)
        return retVal

    def Connect(self, address):
        """Connect to ML-TDR.

        Args:
            address (str): ML-TDR address to connect to.

        Returns:
            bool: Connection success.

        """
        bAddress = bytes(address, encoding="ASCII")     # We pass str as bytes for better marshalling
        self._api.mltdr_detectConnectivity.restype = ctypes.c_int
        retVal = self._api.mltdr_detectConnectivity(self.instance, bAddress)
        return retVal != MLTDRCONNMODE.UNDETECTED
 
    def Configure(self, confparams, result):
        """Configure ML-TDR.

        Args:
            confparams (mltdrConfigurationParams struct) : configuration parameters

            result (double pointer): Array of size 3 (time base, frequency resolution, Sampling Frequency)
        Returns:
            bool: Configuration success.
        """
        self._api.mltdr_configure.restype = ctypes.c_bool
        success = self._api.mltdr_configure(self.instance,confparams, result)
        return success

    def SetSparameterReference(self, sparamRef, RefSize):
        """TDT Calibration (get insertion loss reference circuit).

        Args:
            sparamRef (2D output array (MLMAXCHANNELS * Time Resolution)) : First dimension defines the channel index.
                                                                            Second dimension defines the reference array of each channel.

            RefSize (int): Size of desired captured reference of a channel
        Returns:
            bool: Calibration success.
        """
        self._api.mltdr_getSparameterReference.restype = ctypes.c_int
        retVal = self._api.mltdr_getSparameterReference(self.instance,sparamRef, RefSize)
        return retVal == RefSize

    def CalculateSdd21_C(self, sparamRef, RefSize,  SparamConfig, frequencyValues, S21Values):
        """ Calculate S21 Measurement.

        Args:
            sparamRef (2D input array (MLMAXCHANNELS * RefSize)) : First dimension defines the channel index.
                                                                   Second dimension defines the reference array of each channel.

            RefSize (int): Size of desired captured reference of a channel

            SparamConfig (mltdrSparamConfig struct) : Sparameters configuration

            frequencyValues (double array) : Frequency output array of frequency resolution size

            S21Values (complex 2D output array (MLMAXCHANNELS * Frequency Resolution)) : First dimension defines the channel index.
                                                                                         Second dimension defines the s21 complex array of each channel.

        Returns:
            int: S21 samples count.
        """
        self._api.mltdr_calculateSdd21_C.restype = ctypes.c_int
        retVal = self._api.mltdr_calculateSdd21_C(self.instance,sparamRef, RefSize, SparamConfig, frequencyValues, S21Values)
        return retVal > 0

    def DeEmbedSparam_C(self, freq, S21Values, freq_DeEmbedFile, S21Values_DeEmbedFile):
        """ DeEmbed S21 File.

        Args:
            freq (double array): Frequency input array of measured s21
            
            S21Values (complex array) : S21 complex input array of measured s21
            
            freqSize (int) : s21 freq and values size of measured s21
            
            freq_DeEmbedFile (double array) : Frequency input array of the trace to de-embed
            
            S21Values_DeEmbedFile (complex array): S21 complex input array of the trace to de-embed
            
            freq_DeEmbedFileSize (int) : s21 freq and values size of the trace to de-embed
            
        Returns:
            complex array (size of freq) : S21Values_Output.
        """
        S21Values_Output = (complex * len(freq))()
        self._api.mltdr_DeEmbedSparam_C.restype = ctypes.c_bool
        success = self._api.mltdr_DeEmbedSparam_C(self.instance, freq, S21Values, len(freq), freq_DeEmbedFile, S21Values_DeEmbedFile, len(freq_DeEmbedFile), S21Values_Output)
        if success:
            return S21Values_Output
        else:
            return None

    def SetTx7Taps(self, channels, nChannels, taps):
        """ Set Tx channels 7 taps.

        Args:
            channels (int array): Tx channels input array
            
            taps (int 2D array) : Tx channels Tap values 2D input array (nChannels * 7):
                                  First dimension defines index of channels array.
                                  Second dimension defines taps array of each channel (7 taps per channel).
            
        Returns:
            bool: Function success.
        """
        self._api.mltdr_SetTx7Taps.restype = ctypes.c_bool
        return self._api.mltdr_SetTx7Taps(self.instance, channels, nChannels, taps)

    def CaptureData(self, samplesCount):
        """ Captures and aligns calibrated samples for all channels.

        Args:
            samplesCount (int array): Capture size
            
        Returns:
            5-tuple of:
                - :object:`list` of :object:`double`: xPattern values for all channels of size samplesCount.
                - :object:`list` of :object:`double`: xEye values for all channels of size (samplesCount * 2).
                - :object:`list` of :object:`double`: yEye yValues for all channels of size (samplesCount * 2).
                - :object:`list` of :object:`double`: lockOutput for all channels.
                - :object:`list` of :object:`double`: results for all channels of size CAPTUREOUTPUTSIZE = 11.
                                                      Array output is mapped:
                                                      Min, Max, PeakToPeak, CrossingX, CrossingY, Top, base, LevelZero, LevelOne, LevelTwo, LevelThree

            In case of failure, returns a 5-tuple of None values.                                  
        """
        self._api.mltdr_CaptureData.restype = ctypes.c_bool
        xPattern = (ctypes.c_double * samplesCount * NUMRXCHANNELS)()
        xEye = (ctypes.c_double * (samplesCount * 2) * NUMRXCHANNELS)()
        yValues = (ctypes.c_double * (samplesCount * 2) * NUMRXCHANNELS)()
        lockOutput = (ctypes.c_bool * NUMRXCHANNELS)() 
        results = (ctypes.c_double * CAPTUREOUTPUTSIZE * NUMRXCHANNELS)()  # Data from each channel are linearised sequentially in consecutive blocks of
                                                                           # 11 elements.

        success = self._api.mltdr_CaptureData(self.instance, samplesCount, xPattern, xEye, yValues, lockOutput, results)
        if success:
            output_tuple = (list(xPattern), list(xEye), list(yValues),list(lockOutput), list(results))
            return output_tuple
        return (None,) * 5

    def GetTHD(self, samplesCount, xPattern, yValues, UserDefinedPatternLow, lastHarmonicNumber, thdMode):
        """ Calculates THD.

        Args:
            samplesCount (int array): Capture size
            
            xPattern (double array) : X pattern input array.
            
            yValues (double array): Scope samples input array.

            UserDefinedPatternLow (ulong): User defined pattern generated.

            lastHarmonicNumber (int): Number of the last harmonic to be included in THD calculation.

            thdMode (int):  modes: odd harmonics-even harmonics- odd and even harmonic  (to be included in calculations)

        Returns:
            double: THD value.
        """
        self._api.mltdr_GetTHD.restype = ctypes.c_double
        return self._api.mltdr_GetTHD(self.instance, samplesCount, xPattern, yValues, UserDefinedPatternLow, lastHarmonicNumber, thdMode)

    def GetRLM(self, samplesCount, xPattern, xEye, yValues, initializeData, eyeLevels):
        """ Calculates RLM.

        Args:
            samplesCount (int array): Capture size
            
            xPattern (double array) : X pattern input array.
            
            xEye (double array) : X eye input array.
            
            yValues (double array): Scope samples input array.

            initializeData (bool): Update data.

            eyeLevels (double array): PAM4 eye levels input array in the following order:
                                      currentZeroLevel , currentOneLevel, currentTwoLevel ,currentThreeLevel.
 
        Returns:           
            Output array of 2 elements containing in the following order:
            currentRLM_120D (IEEE 802.3 Annex 120D), currentRLM_94(IEEE 802.3 clause 94).
        """
        self._api.mltdr_GetPAM4RLM.restype = ctypes.c_bool
        output = (ctypes.c_double * 2)()  # Array of size 2 (currentRLM_120D (IEEE 802.3 Annex 120D),
                                          # currentRLM_94(IEEE 802.3 clause94))
        success = self._api.mltdr_GetPAM4RLM(self.instance, samplesCount, xPattern, xEye, yValues, initializeData, eyeLevels, output)
        if success:
            return output
        else:
            return None

    def Get3dBBandwidth(self, Fin_Hz, Sin, IsAbsolute3db, StartFrequencyInHz):
        """ Calculates 3 dB bandwidth.

        Args:            
            Fin_Hz (double array) : Frequency input array.
            
            Sin (complex array) : Sparameters input array.
            
            IsAbsolute3db (bool): true: uses absolute 3dB value, false: uses rolloff from 'StartFrequencyInHz'

            StartFrequencyInHz (double): BW Start Frequency if 'IsAbsolute3db' is false.

        Returns:           
            3db BW double value in Hz.
        """
        self._api.mltdr_Get3dBBandwidth_C.restype = ctypes.c_double
        return self._api.mltdr_Get3dBBandwidth_C(self.instance, Fin_Hz, len(Fin_Hz), Sin, ctypes.c_bool(IsAbsolute3db), ctypes.c_double(StartFrequencyInHz))

    def SetTxOff(self, channels, status):
        """ Sets Tx on/off.

        Args:            
            channels (int array) : channels input array.
                       
            status (bool): true: Sets Tx off; false: Sets Tx on.

        Returns:           
            function success (true if successful, false otherwise).
        """
        self._api.mltdr_SetTxOff.restype = ctypes.c_bool
        return self._api.mltdr_SetTxOff(self.instance, channels, len(channels), status)

    def MeasureScopeVrms(self, xPattern, yValues):
        """ Measure scope Vrms.

        Args:            
            xPattern (double array) : pattern x values.
            
            yValues (double array) : eye y values.

        Returns:           
            Scope Vrms double value.

            Scope Vrms is the device total noise if no input signal is added.
        """
        output = (ctypes.c_double * 1)()  # Vrms value
        self._api.mltdr_MeasureScopeVrms.restype = ctypes.c_bool
        success = self._api.mltdr_MeasureScopeVrms(self.instance, xPattern, yValues, len(xPattern), output)
        if success:
            return output[0]
        else:
            return None

    def GetVrms(self, xPattern, yValues):
        """ Get Vrms.

        Args:            
            xPattern (double array) : pattern x values.
            
            yValues (double array) : eye y values.

        Returns:           
            Vrms double value.
            
            Function requires MeasureScopeVrms in order to measure device noise that will be deEmbedded from Vrms resulted value.
        """
        output = (ctypes.c_double * 1)()  # Vrms value
        self._api.mltdr_GetVrms.restype = ctypes.c_bool
        success = self._api.mltdr_GetVrms(self.instance, xPattern, yValues, len(xPattern), output)
        if success:
            return output[0]
        else:
            return None

    def CheckPRBSLock(self, channel):
        """Check Channel's PRBS lock.

        Args:
            channel (int): Channel index.

        Returns:
            bool: True if channel is locked, False otherwise.

        """
        self._api.mltdr_PRBSVerfication.restype = ctypes.c_bool
        success = self._api.mltdr_PRBSVerfication(self.instance, channel)
        return bool(success)

    def StartRealTimeBER(self):
        """Enable Realtime BER capture mode.
        
        Returns:
            bool: Operation success.
        
        """
        self._api.mltdr_StartRealtimeBER.restype = ctypes.c_bool
        success = self._api.mltdr_StartRealtimeBER(self.instance)
        return bool(success)

    def StopRealTimeBER(self):
        """Disable Realtime BER capture mode.

        Returns:
            bool: Operation success.

        """
        self._api.mltdr_StopRealtimeBER.restype = ctypes.c_bool
        success = self._api.mltdr_StopRealtimeBER(self.instance)
        return bool(success)

    def ReadRealTimeBER(self):
        """Read Realtime BER values.

        Returns:
            3-tuple of:
                - :object:`list` of :object:`int`: Capture duration in ms for all channels.
                - :object:`list` of :object:`double`: BER values for all channels.
                - :object:`list` of :obj:`int`: Error count list for all channels

            In case of failure, returns a 3-tuple of None values.

        """
        self._api.mltdr_ReadRealtimeBER.restype = ctypes.c_bool
        time_spent_ms = (ctypes.c_uint32 * NUMTXCHANNELS)()
        error_counts = (ctypes.c_uint64 * NUMTXCHANNELS)()
        ber_values = (ctypes.c_double * NUMTXCHANNELS)()
        success = self._api.mltdr_ReadRealtimeBER(self.instance, time_spent_ms, error_counts, ber_values)
        if success:
            output_tuple = (list(time_spent_ms), list(ber_values), list(error_counts))
            return output_tuple
        print("ReadRealTimeBER: %s" % success)
        #  Failed! 
        return (None,) * 3

    def ReadBerMSBLSB(self):
        self._api.mltdr_ReadBerMSBLSB.restype = ctypes.c_bool
        errorCountsMSB = (ctypes.c_uint64 * NUMTXCHANNELS)()
        errorCountsLSB = (ctypes.c_uint64 * NUMTXCHANNELS)()

        success = self._api.mltdr_ReadBerMSBLSB(self.instance, errorCountsMSB, errorCountsLSB)
        if success:
              output_tuple = (list(errorCountsMSB), list(errorCountsLSB))
              return output_tuple
        print("ReadBerMSBLSB: %s" % success)
        #  Failed! 
        return (None,) * 2

    def ReadRxFFETaps(self, channel):
        """Read Rx FFE Taps.

        Returns:
            List of 15 Rx FFE tap values.

            In case of failure, returns an empty list.

        """
        self._api.mltdr_GetFFE.restype = ctypes.c_bool
        taps = (ctypes.c_double * 15)()
        success = self._api.mltdr_GetFFE(self.instance, channel, taps)
        if success:
            return list(taps)
        #  Failed! 
        return []
    
    def CheckTxLock(self, channel):
        """Check Channel's Tx lock.

        Args:
            channel (int): Channel index.

        Returns:
            bool: True if channel is locked, False otherwise.

        """
        self._api.mltdr_ReadTXLock.restype = ctypes.c_bool
        status= (ctypes.c_bool*1)()
        success = self._api.mltdr_ReadTXLock(self.instance, channel, status)
        return bool(success and status)

    def Deskew(self):
        """Deskew signals at the DUT level before measuring impedance.

        Args:
            offset (int pointer): Deskew offset output array of size MLMAXCHANNELS
        Returns:
            bool: Function success.

        Remarks:
            DUT should be removed before calling this function. Deskew is done to precisely align the signals at the DUT.
        """

        output = (ctypes.c_int * MAXCHANNELS)()  # Vrms value
        self._api.mltdr_Deskew.restype = ctypes.c_bool
        success = self._api.mltdr_Deskew(self.instance, output)
        return output

    def ReadTDRValues(self, params,samplesCount):
        """ Reads tdr Values for all channels.

        Args:
            confparams (MLTDRPARAMS struct) : tdr settings
            samplesCount (int): Capture size

        Returns:
            2-tuple of:
                - :object:`list` of :object:`double`: Normalized Impedance values for all channels of size samplesCount.
                - :object:`list` of :object:`double`: Ohm Impedance values for all channels of size samplesCount.

            In case of failure, returns a 2-tuple of None values.                                  
        """
        self._api.mltdr_ReadTDRValues.restype = ctypes.c_int
        tdr = (ctypes.c_double * samplesCount * MAXCHANNELS)()
        tdrOhm = (ctypes.c_double * samplesCount * MAXCHANNELS)()
        datasize = (ctypes.c_int * MAXCHANNELS)()

        ret = self._api.mltdr_ReadTDRValues(self.instance, params, tdr, tdrOhm, datasize)
        if ret>0:
            output_tuple = ([],[])
            for i in range(MAXCHANNELS):    
                output_tuple[0].append(tdr[i][:datasize[i]])
                output_tuple[1].append(tdrOhm[i][:datasize[i]])
            return output_tuple
        return (None,) * 2

    def CalculateXValues(self, SamplesCount, velocityFactor):
        """ Calculates x values.

        Args:
            resolution (int): Capture size
            velocityFactor (double): Velocity factor: The ratio of actual speed to the speed of light -> between 0 and 1

        Returns:
            2-tuple of:
                - :object:`list` of :object:`double`: X values output array in ps of size SamplesCount.
                - :object:`list` of :object:`double`: X values output array in m of size SamplesCount.

            In case of failure, returns a 2-tuple of None values.                                  
        """
        self._api.mltdr_CalculateXValues.restype = ctypes.c_bool
        xps = (ctypes.c_double * SamplesCount)()
        xm = (ctypes.c_double * SamplesCount)()

        success = self._api.mltdr_CalculateXValues(self.instance, SamplesCount, ctypes.c_double(velocityFactor), xps, xm)
        if success:
            output_tuple = (list(xps), list(xm))
            return output_tuple
        return (None,) * 2
